#!/usr/bin/env python
# # -*- coding:iso-8859-1  -*-

"""
 Name:	      graph_gel_degel_area.py
 Description: Prend un fichier avec des entrees de la forme:
  iso8601, [G|D], [0|1], area
  et fait un graphique avec pour chaque point les coordonnees (G,D).
 
 
 Notes: 

 Author: Miguel Tremblay
 Date: 25 mai 2006
"""

import sys
from rpy import r
import os.path
import numarray

def main(sFilepath, sPathGraph):
    dGelDegelArea = { 'G':[],
                      'D':[] }

    # Ouverture du fichier texte
    file_input = open(sFilepath, 'r')
    sFilename = os.path.split(sFilepath)[1] 
    [sStation, sYear, spwet, spwet2] = sFilename.split('_')

    # Retrouve le nombre de ligne
    lLinesInFile = file_input.readlines()

    # On regarde si la premiere ligne est un gel ou un degel
    #  Si c'est un gel, on l'enleve
    if lLinesInFile[0].split(',')[1] == 'G':
        lLinesInFile = lLinesInFile[1:]

    # On recupere les aires et on en profite pour si on a
    #  2 'G' ou 2 'D' de suite
    sLastGelDegel = 'G'
    for sLine in lLinesInFile:
        lLineSplit = sLine.split(',')
        sGelDegel = lLineSplit[1]
        fArea = float(lLineSplit[-1])
        if sGelDegel == sLastGelDegel:
            print "Erreur, 2 valeurs de suites:",sLastGelDegel
            print "dans le fichier", sFilepath
            continue
            sys.exit(1)
        else:
            sLastGelDegel = sGelDegel
            dGelDegelArea[sGelDegel].append(fArea)

    # Verification si le nombre de gel et de degel correspondent
    if len(dGelDegelArea['G']) >  len(dGelDegelArea['D']):
        dGelDegelArea['G'] = dGelDegelArea['G'][:-1]
    elif len(dGelDegelArea['D']) >  len(dGelDegelArea['G']):
        dGelDegelArea['D'] = dGelDegelArea['D'][:-1]

    draw_graph(sPathGraph, sStation, sYear, dGelDegelArea)

def draw_graph(sPathGraph, sStation, sYear, dGelDegelArea):
    """
    Dessine une graphique qui met un point de l'aire du gel en fonction
    de l'air du degel.
    """
    # Label
    sMain = " %s :: %s" %\
            (sStation, sYear)
    sXLabel = "Aire de gel"
    sYLabel = "Aire de dgel"

    lAxisRange = [-100, 1000]
    
    lAxeX = numarray.array(dGelDegelArea['G'])
    lAxeY = dGelDegelArea['D']

    # Mettre les valeurs maximales et minimales 
    for lAxe in [lAxeX,lAxeY]:
        for i in range(0, len(lAxe)):
            if lAxe[i] < lAxisRange[0]:
                lAxe[i] = lAxisRange[0]
            elif lAxe[i] > lAxisRange[1]:
                lAxe[i] = lAxisRange[1]
    
    print lAxeY
    
    r.png(sPathGraph, width = 480, height = 480,
         pointsize = 12, bg = "white")

    r.par(las=1)
    r.plot(lAxeX, lAxeY, yaxs='i', xaxs='i', \
           ylim=lAxisRange, xlim=lAxisRange, pch='O', cex=0.5,
           type='p', xlab=sXLabel, ylab=sYLabel,
           main=sMain,# xaxt='n', yaxt='n',\
           col='blue',  lty=0, xpd=r.NA)

    print 'Writing', sPathGraph
    r.dev_off()

def draw_legend(dStation, dCouleur):
    """
    Draw the legend for the graph. dStation contains the string
    of the filename before the first '_' and dCouleur contains the
    color for the data of a specific filenames.
    """
    nNbrStation = len(dStation.values())
    nHalfValue = nNbrStation/2

    lFullKeys = dStation.keys()
    lFirstHalfKeys = lFullKeys[0:nHalfValue]
    lSecondHalfKeys = lFullKeys[nHalfValue:]

    lCouleur = []
    lTextLegend = []
    for sKey in lFirstHalfKeys:
        lCouleur.append(dCouleur[sKey])
        lTextLegend.append(dVilles[dStation[sKey]])

    r.legend(1, 199, lTextLegend, lwd=2, col=lCouleur,\
             cex=0.8)

    lCouleur = []
    lTextLegend = []
    for sKey in lSecondHalfKeys:
        lCouleur.append(dCouleur[sKey])
        lTextLegend.append(dVilles[dStation[sKey]])

    r.legend(5, 199, lTextLegend, lwd=2, col=lCouleur,\
             cex=0.8)
    
def start_graph(sPngPath):
    """
    Debute le graphique en dissante le cadre.
    """

    # String pour le graphique
    s_x_label = 'Mois'
    s_y_label = 'Prcipitation moyenne (mm)'
    s_main = 'Climatologie du nombre de mm \n de prcipitation liquide par mois'
    
    # L'axe des X ira de 6-12 1-5
    lAxeX =  range(6,13) + range(1,6)
    lAxeY = range(0,200) # cet axe Y est simplement pour avoir un domaine
    r.png(sPngPath, width = 480, height = 480,
         pointsize = 12, bg = "white")
    r.par(las=1)
    r.plot(lAxeX, lAxeX, yaxs='i', xaxs='i', \
           ylim=[min(lAxeY), max(lAxeY)], \
           type='l', xlab=s_x_label, ylab=s_y_label,
           main=s_main, xaxt='n', yaxt='n',\
           col='blue',  lty=0)
    lXtick = range(1,13)
    lYtick = range(0,200,25)
    r.axis(1, lOrdreMois, at=lXtick)
    r.axis(2, at=lYtick)
    draw_grid()


def draw_grid():
    """
    If dx and/or dy is set, put a gray line on the coorresponding
    axes separated by this value.
    """
    # Draw vertical line
    rlUsr = r.par('usr')
    rYdomain =r.c(rlUsr[2], rlUsr[3]) 
    for i in range(2,12):
        rXdomain = r.c(i, i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Draw horizontal line
    rXdomain = r.c(rlUsr[0], rlUsr[1])
    for i in range(25,199,25):
        rYdomain = r.c(i,i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Redraw extremities
    r.lines(r.c(rlUsr[0], rlUsr[0]), \
            rYdomain, col='black')
    r.lines(r.c(rlUsr[1], rlUsr[1]), \
            rYdomain, col='black')


if __name__ == "__main__":


    if len(sys.argv) < 3:
        print "Usage graph_gel_degel_area.py fichier.txt graph.png"
        sys.exit(1)


    sFilename = sys.argv[1]
    sPathGraph = sys.argv[2]
    if 'png' not in sPathGraph:
        print "Last argument is png path!:", sPathGraph
        sys.exit(1)

    main(sFilename, sPathGraph)




