#!/usr/bin/python
# -*- coding:iso-8859-1  -*-
#
# Code utilise pour l'article http://ptaff.ca/routes/
#
# Copyright (C) 2006 Miguel Tremblay
#
#  Questions or bugs report: miguel.tremblay@ptaff.ca
#
#
# Code contributed by:
#  Miguel Tremblay - ptaff.ca
#
########################################################################
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#


"""
 Name:	      nombre_de_degel.py
 Description: Calcule le nombre de fois que la temperature passe d'en
  dessous de zero a au-dessus de zero. Pour qu'un compte soit fait,
  il faut que la temperature passe de -1 a +1. Les oscillations entre
  ces 2 valeurs ne sont pas compte.
  2 valeurs seuils en argument. Seul les evenements de gel et degel avec
  une aire dans ce seuil sont compts.
 
 Notes: Le script est fait pour fonctionner avec un repertoire ou
  chaque fichier correspond a un mois d'une annee.

 Author: Miguel Tremblay
 Date: 12 aout 2006
"""

import sys
import os
import os.path

#sHeader = 'YYYY-MM, nombre_de_degel, nombre_de_degel_avec_pluie, nombre_observations\n'

def main(sDirectory, sYear, fSeuilAireMin, fSeuilAireMax):


    lFilenames = os.listdir(sDirectory)
    lThisYearFilenames = []
    for sFilename in lFilenames:
        if sYear in sFilename:
            lThisYearFilenames.append(sDirectory + sFilename)

    if len(lThisYearFilenames) == 0:
        sMessage =  "No files for year '%s' in directory '%s'" % \
                   (sYear, sDirectory)
        print sMessage
        sys.exit(1)
            
    lDate_nbr_degel = []
    sSplit_underscore = sFilename.split('_')
    sStation = sSplit_underscore[0]
    (lTexteFinal,  nNbrObs)  = \
                  get_nbr_degel(lThisYearFilenames, \
                                fSeuilAireMin, fSeuilAireMax)

    # Write the output
    sOutput = os.path.split(sDirectory[:-1])[0] + '/degel_2_seuils/' +\
              sStation +  '_' + sYear + '_nbr_degel_seuil_min_' +\
              str(int(fSeuilAireMin)) +\
              '_max_' +     str(int(fSeuilAireMax))   + '.txt'
    print "Writing: ", sOutput
    file_output = open(sOutput, 'w')
    file_output.writelines(lTexteFinal[:-1])

def get_nbr_degel(lFilenames, fSeuilAireMin, fSeuilAireMax):    
    """
    Calcule le nombre de fois que la temperature passe au-dessus de 1 et que
    l'aire par la suite est suprieure  un seuil.

    Input: Nom du fichier
    Output: (nbr_degel, nbr_degel_mouille, nbr_observations)
    
    """
    fSeuilUp = 1.0
    fSeuilDown = -1.0
    
    # Ouvrir le fichier
    lOriginalFiles = []
    for sFilename in lFilenames:
        sOriginalFile = open(sFilename, 'r')
        lOriginalFile = sOriginalFile.readlines()
        # Verifier que la presente version est la bonne.
        if len(lOriginalFile[0].split(',')) != 5:
            print "Ce fichier n'est pas a jour:", sFilename
            sys.exit(1)        
        lOriginalFiles = lOriginalFiles + lOriginalFile

    nNbr_observations = len(lOriginalFiles)
    # Parsing du fichier
    lTemperature = []
    lMouille = []
    lArea = []
    lDate = []
    lOriginalFiles.sort()
    for sLine in lOriginalFiles:
        lLine = sLine.split(',')
        lDate.append(lLine[0])
        lTemperature.append(float(lLine[1]))
        lMouille.append(lLine[-2])
        lArea.append(float(lLine[-1]))

    # Est-ce que l'on cherche en premier une temperature au dessus de 1?
    lTexteFinal = []
    if lTemperature[0] < 0:
        b_look_over_1 = True
        lTexteFinal.append(lDate[0] + ',G,')
    # ou en dessous de -1 ?
    else:
        b_look_over_1 = False
        lTexteFinal.append(lDate[0] + ',D,')

    n_nbr_input = len(lOriginalFiles)
    n_nbr_degel = 0

    i = 1
    bPluiePresente = False
    while i < n_nbr_input:
        # On cherche le degel (on est en periode de gel)
        if b_look_over_1:
            fArea = 0.0
            # Si la temperature est en bas du seuil
            while  i < n_nbr_input and lTemperature[i] < fSeuilUp:
                i = i + 1
            # si l'aire n'est pas
            #  suffisement grande, on continue
            while  i < n_nbr_input and fArea <= fSeuilAireMin :
#                print fArea
                if  lArea[i] > 0:
                    fArea = fArea + lArea[i]
                elif lTemperature[i] < fSeuilUp : # reset
                    fArea = 0
                i = i + 1
#            print fArea
            if i == n_nbr_input:
                break
            elif fArea <=  fSeuilAireMax:
                lTexteFinal[n_nbr_degel] = lTexteFinal[n_nbr_degel] + \
                                           '0,' + str(fArea) + '\n'
                lTexteFinal.append(lDate[i]+  ',D,')
                n_nbr_degel = n_nbr_degel + 1
                b_look_over_1 = False
            elif fArea > fSeuilAireMax:
                b_look_over_1 = False
                continue
            else:
                break
        # On cherche le gel (on est en periode de degel)
        else:
            fArea = 0.0
            while  i < n_nbr_input and lTemperature[i] > fSeuilDown: 
                if not bPluiePresente and lMouille[i] == '1':
                    bPluiePresente = True
                i = i + 1
            while  i < n_nbr_input and fArea <= fSeuilAireMin:
#                print lArea[i], fArea
                if  lArea[i] < 0:
                    fArea = fArea - lArea[i]
                elif lTemperature[i] > fSeuilDown : # reset
                    fArea = 0                
                if not bPluiePresente and lMouille[i] == '1':
                    bPluiePresente = True
                i = i + 1
            if i == n_nbr_input:
                break
            if fArea <= fSeuilAireMax:
                if bPluiePresente:
                    sPluiePresente = '1'
                else:
                    sPluiePresente = '0'
                
                lTexteFinal[n_nbr_degel] = lTexteFinal[n_nbr_degel] + \
                                           sPluiePresente + ',' +\
                                           str(fArea) + '\n'
                lTexteFinal.append(lDate[i]+  ',G,')                
                n_nbr_degel = n_nbr_degel + 1
                b_look_over_1 = True
                bPluiePresente = False
            elif fArea > fSeuilAireMax:
                b_look_over_1 = True
                continue
            else:
                break

    # Patch 
    if n_nbr_degel == 0:
        n_nbr_degel_mouille = 0
    
    return (lTexteFinal, nNbr_observations )

if __name__ == "__main__":

    lArgs = ['']*1

    if len(sys.argv) < 5:
        print "Usage nombre_de_degel.py directory year seuil_aire_min seuil_aire_max"
        sys.exit(1)
        
    sDirectory = sys.argv[1]
    sYear = sys.argv[2]
    fSeuilAireMin = float(sys.argv[3])
    fSeuilAireMax = float(sys.argv[4])
    
    main(sDirectory, sYear, fSeuilAireMin, fSeuilAireMax)


