#!/usr/bin/python
# -*- coding:iso-8859-1  -*-
#
# Code utilise pour l'article http://ptaff.ca/routes/
#
# Copyright (C) 2006 Miguel Tremblay
#
#  Questions or bugs report: miguel.tremblay@ptaff.ca
#
#
# Code contributed by:
#  Miguel Tremblay - ptaff.ca
#
########################################################################
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#

"""
 Name:	       xml2txt.py
 Description: Transform a xml file of hourly data from EC
              in a text file in the ISO 8601 format and with the
              air temperature:
              YYYY-MM-DDTHH:mm, TT
 
 Notes: 

 Author: Miguel Tremblay
 Date: 12 mai 2006
"""

import sys
import os.path
import os
from datetime import datetime
from datetime import timedelta

def main(sFilename):

    # Ouvrir le fichier 
    sOriginalFile = open(sFilename, 'r')

    # Parsing for the output path
    [sBasename, sFile]  = os.path.split(sFilename)    
    sFilename_output = sFile[:-3]  + 'txt'
    sNewBasename = os.path.split(sBasename)[0] + '/txt/'
    if not os.path.exists(sNewBasename):
        print 'Creating:',sNewBasename
        os.mkdir(sNewBasename)
    sPathOutput = sNewBasename + sFilename_output
    
    lTotalFile = sOriginalFile.readlines()

    # Fetch the dates
    lxml_date = [sDate for sDate in lTotalFile if '<stationdata' in sDate]
    # Fetch the temperature
    lxml_temperature =  [sTemperature for sTemperature in \
                         lTotalFile if 'rature' in sTemperature]
    # Fetch the Etat du temps
    l_etat_temps = [sEtat for sEtat in \
                    lTotalFile if '<weather' in sEtat]

    # Check if there is the same number of dates and of temperatures
    if len(lxml_temperature) != len (lxml_date):
        sErrorMessage = "Il n'y a pas autant de temperature que de dates:" +\
                        " temp: %s  date %s" % ( len(lxml_temperature),\
                                                 len(lxml_date))
        print sErrorMessage
        sys.exit(1)

    l_date_temperature = []
    l_date_iso8601 = []
    l_temperature = []
    for i in range(0,len(lxml_date)):
        s_iso_8601 = get_date_in_xml(lxml_date[i])
       
        s_temperature = get_element_value(lxml_temperature[i], True)
        s_etat = get_element_value(l_etat_temps[i])
        b_pluie = y_mouille_tu(s_etat)
        
        s_date_temperature = s_iso_8601 + ',' + s_temperature +\
                             ',' + s_etat + ',' + str(b_pluie) 
        # QA from an invalid data in hly5889_1995_1.xml
        if 'ature' not in s_date_temperature :
            l_date_temperature.append(s_date_temperature)
            l_date_iso8601.append(s_iso_8601)
            l_temperature.append(float(s_temperature))
 
    # Calcule l'aire entre chaque heure
    for i in range(0,len(l_date_iso8601)-1):
        # Premiere valeur
        sDateISO8601_1 = l_date_iso8601[i]
        nTemperature_1 = l_temperature[i]
        nYear_1 = int(sDateISO8601_1[0:4])
        nMonth_1 = int(sDateISO8601_1[5:7])
        nDay_1 = int(sDateISO8601_1[8:10])
        nHour_1 = int(sDateISO8601_1[11:13])
        nMinute_1 = int(sDateISO8601_1[14:16])
        datetime1 = datetime(nYear_1, nMonth_1, nDay_1,\
                             nHour_1, nMinute_1)
        # Deuxieme valeur
        sDateISO8601_2 = l_date_iso8601[i+1]
        nTemperature_2 = l_temperature[i+1]
        nYear_2 = int(sDateISO8601_2[0:4])
        nMonth_2 = int(sDateISO8601_2[5:7])
        nDay_2 = int(sDateISO8601_2[8:10])
        nHour_2 = int(sDateISO8601_2[11:13])
        nMinute_2 = int(sDateISO8601_2[14:16])
        datetime2 = datetime(nYear_2, nMonth_2, nDay_2,\
                             nHour_2, nMinute_2)
        # Compute the difference
        fHour = (datetime2-datetime1).seconds/3600.0
        fArea = fHour * (nTemperature_1+nTemperature_2)/2
        l_date_temperature[i] = l_date_temperature[i] + ',' + str(fArea) + '\n'

    # Write the output
    if len(l_date_temperature) == 0 :
        print "No observation in", sFilename
        return
    file_output = open(sPathOutput, 'w')
    file_output.writelines(l_date_temperature)
    sMessage =  'Writing %s' % (sPathOutput)
    print sMessage

def y_mouille_tu(sEtat):
    """
    Regarder si l'etat peut etre considerer comme etant de la precipitation.
    La liste des etats peut etre trouve ici:
    http://www.climate.weatheroffice.ec.gc.ca/Glossary-popup_f.html#weather
    
    Note: on va travailler tout en lowercase pour s'eviter du trouble.
    """
    lEtatMouille = ['tornade', 'trombe marine', 'orages',  'pluie', \
                    'averses', 'bruine',  'neige','cristeaux de glace',\
                    'granules', 'grle', 'prcipitation']

    sEtatLowercase = sEtat.lower()
    
    for sEtatMouille in lEtatMouille:
        if sEtatMouille in sEtatLowercase:
            return 1
    return 0
        

def get_element_value(sXML_string, b_number=False):
    """
    Input: xml string like 
    <temp units="C" description="Temprature" flag=" ">-16,0</temp>
    Output: string whit only the value in the element
    """
    nIndiceStart = sXML_string.find('">')+2
    nIndiceStop = sXML_string.find('</')

    # Get the value and replace ',' by a '.'
    sValue = sXML_string[nIndiceStart:nIndiceStop].strip()
    if b_number:
        sValue = sValue.replace(',','.')
    else:
        sValue = sValue.replace(',',' -- ')
    return sValue
    
                   
def get_date_in_xml(sXML_string):
    """
    Input: xml string like 
    <stationdata year="1994" month="1" day="1" hour="0" minute="0">
    Output: ISO 8601 string like YYYY-MM-DDTHH:mm
    """
    lDateString = ['year', 'month', 'day', 'hour', 'minute']
    #Get the date
    lDate = []
    for sDate in lDateString:
        nIndiceStart = sXML_string.find(sDate) + len(sDate) + 2
        nIndiceEnd = sXML_string[nIndiceStart:].find('"') + nIndiceStart
        sAttributeValue = sXML_string[nIndiceStart:nIndiceEnd]
        # Add '0' if the string lenght is less than 2
        if len(sAttributeValue) < 2:
            sAttributeValue = '0' + sAttributeValue
        lDate.append(sAttributeValue)
            
    sYYYY_MM_DDTHH_mm = lDate[0] + '-' + lDate[1] + '-' + lDate[2] +\
                        'T' + lDate[3] + ':' + lDate[4]

    return sYYYY_MM_DDTHH_mm

if __name__ == "__main__":

    lArgs = ['']*1

    if len(sys.argv) < 2:
        print "Usage xml2txt.py file.xml"
        sys.exit(1)
        

    sFilename = sys.argv[1]

    
    main(sFilename)



